/** 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 *  
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *  
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
 * 
 * @version 2.0
 * DisChoco
 * SNPanel.java
 */ 
package benchmark.meetingscheduling.gui;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.HashMap;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;

import dischoco.DisChoco;
import benchmark.random.RBDisCSPGenerator;
import gui.visualizing.Edge;
import gui.visualizing.Graph;
import gui.visualizing.GraphPanel;
import gui.visualizing.TreeDrawing;
import gui.visualizing.Vertex;
import kernel.order.AgentComplexity;
import kernel.order.LexicographicAO;
import kernel.order.MaxDegPlusMinDomAO;
import kernel.order.MinDomDegAO;
import kernel.order.RandomAO;
import kernel.order.TreeAO;
import util.Commands;
import util.graph.PriorityTree;
import util.parser.InstanceParser;
import static gui.Gui.*;

/**
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 * 
 */

public class MeetingSchedulingPanel extends JPanel {

	public static final String fixKeys = P_V;
	
	private static final long serialVersionUID = -6560016016508986831L;
	private JLabel typeLabel = null;
	private JLabel prepocessingLabel = null;
	private JLabel randomDCSPLabel = null;
	private JLabel nbMeetLabel = null;
	private JLabel nbMeetingsLabel = null;
	private JLabel nbAgLabel = null;
	private JLabel nbAgentsLabel = null;
	private JLabel nbMPALabel = null;
	private JLabel nbMeetingPerAgentLabel = null;
	private JLabel dispALabel = null;
	private JLabel disponibilityOfAgentsLabel = null;
	private JLabel nbDLabel = null;
	private JLabel nbDaysLabel = null;
	private JLabel nbHLabel = null;
	private JLabel nbHoursLabel = null;
	private JPanel westPanel = null;
	private JPanel eastPanel = null;
	private JLabel nbXLabel = null;
	private JLabel nbXAxisLabel = null;
	private JLabel nbYLabel = null;
	private JLabel nbYAxisLabel = null;
	private JComboBox agentsOrdering = null;
//	private File problem = null;
	private HashMap<Integer, AgentComplexity> complexities = null;
	private Graph<Vertex, Edge> graph = null;
	private JPanel graphPanel = null;
	private Dimension size = new Dimension(760, 400);  //  @jve:decl-index=0:
	private final static String TreeAO_PANEL = "TreeAO_Panel";
	private final static String GraphAO_PANEL = "GraphAO_Panel";
	private static final int WEST_WIDTH = 360;
	private int nbMeetings;
	private int nbAgents;
	private int nbMeetingsPerAgent;
	private int disponibilityOfAgents;
	private int nbDays;
	private int nbHours;
	private int xaxis;
	private int yaxis;
	/**
	 * 
	 * @param file
	 * @param d
	 */
	public MeetingSchedulingPanel(Dimension d) {
		super();
		size = d;
		setSize(size);
		setPreferredSize(size);
		String fileName = DisChoco.filePath.getAbsolutePath();
		InstanceParser parser = new InstanceParser(fileName);
		parser.loadInstance(fileName);
		parser.parse(false);
		parser.convertToSimpleModel();
		complexities = parser.getAgentsComplexity();
		graph = parser.getGraph(new Dimension(650, 435));
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		if (fileName.endsWith("xml.bz2")) {
			try {
				Process p = Runtime.getRuntime().exec("bunzip2 -c " + fileName);
				document = sxb.build(p.getInputStream());
				p.waitFor();
				p.exitValue();
				p.destroy();
			} catch (Exception e) {
				System.out.println("Problem with " + fileName);
				e.printStackTrace();
			}
		}
		try { document = sxb.build(DisChoco.filePath);
		} catch (Exception e) {}
		Element gui = document.getRootElement().getChild(RBDisCSPGenerator.GuiPresentation);
		setPresentation(gui);
		setLayout(new BorderLayout());
		add(getWestPanel(), BorderLayout.WEST);
		add(getEastPanel(), BorderLayout.CENTER);
	}

	public void setPresentation(Element presentation) {
		this.setLayout(null);
		this.nbMeetings = new Integer(presentation.getAttributeValue(NB_MEETINGS));
		this.nbAgents = new Integer(presentation.getAttributeValue(NB_AGENTS));
		this.nbMeetingsPerAgent = new Integer(presentation.getAttributeValue(NB_MEETINGS_PER_AGENT));
		this.disponibilityOfAgents = new Integer(presentation.getAttributeValue(DISPONIBILITY));
		this.nbDays = new Integer(presentation.getAttributeValue(NB_DAYS));
		this.nbHours = new Integer(presentation.getAttributeValue(NB_HOURS));
		this.xaxis = new Integer(presentation.getAttributeValue(X_axis));
		this.yaxis = new Integer(presentation.getAttributeValue(Y_axis));
	}

	/**
	 * This method initializes westPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getWestPanel() {
		if (westPanel == null) {
			westPanel = new JPanel(null);
			Dimension d = new Dimension(WEST_WIDTH, size.height);
			westPanel.setMinimumSize(d);
			westPanel.setMaximumSize(d);
			westPanel.setPreferredSize(d);
			prepocessingLabel = new JLabel("Prepocessing Ordering", JLabel.CENTER);
			prepocessingLabel.setBounds(new Rectangle(80, 15, 200, 22));
			typeLabel = new JLabel("Type :", JLabel.RIGHT);
			typeLabel.setBounds(new Rectangle(25, 80, 150, 22));
			randomDCSPLabel = new JLabel(Commands.MeetingScheduling, JLabel.LEFT);
			randomDCSPLabel.setBounds(new Rectangle(190, 80, 150, 22));
			nbMeetLabel = new JLabel(NumberOfMeetings, JLabel.RIGHT);
			nbMeetLabel.setBounds(new Rectangle(25, 115, 150, 22));
			nbMeetingsLabel = new JLabel("", JLabel.LEFT);
			nbMeetingsLabel.setBounds(new Rectangle(190, 115, 150, 22));
			nbAgLabel = new JLabel(NumberOfAgents, JLabel.RIGHT);
			nbAgLabel.setBounds(new Rectangle(25, 150, 150, 22));
			nbAgentsLabel = new JLabel("", JLabel.LEFT);
			nbAgentsLabel.setBounds(new Rectangle(190, 150, 150, 22));
			nbMPALabel = new JLabel(NumberOfMeetingsPerAgent, JLabel.RIGHT);
			nbMPALabel.setBounds(new Rectangle(25, 185, 150, 22));
			nbMeetingPerAgentLabel = new JLabel("", JLabel.LEFT);
			nbMeetingPerAgentLabel.setBounds(new Rectangle(190, 185, 150, 22));
			dispALabel = new JLabel(DisponibilityOfAgents, JLabel.RIGHT);
			dispALabel.setBounds(new Rectangle(25, 220, 150, 22));
			disponibilityOfAgentsLabel = new JLabel("", JLabel.LEFT);
			disponibilityOfAgentsLabel.setBounds(new Rectangle(190, 220, 150, 22));
			nbDLabel = new JLabel(NumberOfDays, JLabel.RIGHT);
			nbDLabel.setBounds(new Rectangle(25, 255, 150, 22));
			nbDaysLabel = new JLabel("", JLabel.LEFT);
			nbDaysLabel.setBounds(new Rectangle(190, 255, 150, 22));
			nbHLabel = new JLabel(NumberOfHours, JLabel.RIGHT);
			nbHLabel.setBounds(new Rectangle(25, 290, 150, 22));
			nbHoursLabel = new JLabel("", JLabel.LEFT);
			nbHoursLabel.setBounds(new Rectangle(190, 290, 150, 22));
			nbXLabel = new JLabel(NumberOfXAxis, JLabel.RIGHT);
			nbXLabel.setBounds(new Rectangle(25, 325, 150, 22));
			nbXAxisLabel = new JLabel("", JLabel.LEFT);
			nbXAxisLabel.setBounds(new Rectangle(190, 325, 150, 22));
			nbYLabel = new JLabel(NumberOfYAxis, JLabel.RIGHT);
			nbYLabel.setBounds(new Rectangle(25, 360, 150, 22));
			nbYAxisLabel = new JLabel("", JLabel.LEFT);
			nbYAxisLabel.setBounds(new Rectangle(190, 360, 150, 22));
			westPanel.add(prepocessingLabel);
			westPanel.add(getAgentsOrdering());
			westPanel.add(typeLabel);
			westPanel.add(randomDCSPLabel);
			westPanel.add(nbMeetLabel);
			westPanel.add(nbMeetingsLabel);
			westPanel.add(nbAgLabel);
			westPanel.add(nbAgentsLabel);
			westPanel.add(nbMPALabel);
			westPanel.add(dispALabel);
			westPanel.add(nbMeetingPerAgentLabel);
			westPanel.add(disponibilityOfAgentsLabel);
			westPanel.add(nbDLabel);
			westPanel.add(nbDaysLabel);
			westPanel.add(nbHLabel);
			westPanel.add(nbHoursLabel);
			westPanel.add(nbXLabel);
			westPanel.add(nbXAxisLabel);
			westPanel.add(nbYLabel);
			westPanel.add(nbYAxisLabel);
			nbMeetingsLabel.setText(this.nbMeetings + "");
			nbAgentsLabel.setText(this.nbAgents + "");
			nbMeetingPerAgentLabel.setText(this.nbMeetingsPerAgent + "");
			disponibilityOfAgentsLabel.setText(this.disponibilityOfAgents + "");
			nbDaysLabel.setText(this.nbDays + "");
			nbHoursLabel.setText(this.nbHours + "");
			nbXAxisLabel.setText(this.xaxis + "");
			nbYAxisLabel.setText(this.yaxis + "");
		}
		return westPanel;
	}

	public JComboBox getAgentsOrdering() {
		if (agentsOrdering == null) {
			agentsOrdering = new JComboBox();
			agentsOrdering.setBounds(new Rectangle(100, 45, 150, 22));
			agentsOrdering.addItem(Commands.LexicoAO);
			agentsOrdering.addItem(Commands.RandomAO);
			agentsOrdering.addItem(Commands.MinDomDegAO);
			agentsOrdering.addItem(Commands.MaxDegPlusMinDomAO);
			agentsOrdering.addItem(Commands.TreeAO);
			agentsOrdering.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if (e.getStateChange() == ItemEvent.SELECTED) {
						plotEastPanel((String) e.getItem());
					}
				}
			});
			System.out.println("Agent ordering : " + LexicographicAO.buildpriorityTree(complexities));
			plotGraphAO();
		}
		return agentsOrdering;
	}

	/**
	 * This method initializes figurePanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getEastPanel() {
		if (eastPanel == null) {
			eastPanel = new JPanel(new CardLayout());
			Dimension d = new Dimension(size.width-WEST_WIDTH, size.height);
			eastPanel.setPreferredSize(d);
		}
		return eastPanel;
	}

	/**
	 * This method initializes randomAOPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getGrpahPanel() {
		if (graphPanel == null) {
			graphPanel = new GraphPanel(graph, new Dimension(size.width-WEST_WIDTH, size.height));
		}
		return graphPanel;
	}
	/**
	 * 
	 * @param selectedItem
	 */
	private void plotEastPanel(String selectedItem) {
		if (selectedItem.equals(Commands.TreeAO)) {
			plotTreeAO();
		} else if (selectedItem.equals(Commands.LexicoAO)) {
			System.out.println("Agent ordering : " + LexicographicAO.buildpriorityTree(complexities));
			plotGraphAO();
		} else if (selectedItem.equals(Commands.RandomAO)) {
			System.out.println("Agent ordering : " + RandomAO.buildpriorityTree(complexities));
			plotGraphAO();
		} else if (selectedItem.equals(Commands.MinDomDegAO)) {
			System.out.println("Agent ordering : " + MinDomDegAO.buildpriorityTree(complexities));
			plotGraphAO();
		} else if (selectedItem.equals(Commands.MaxDegPlusMinDomAO)) {
			System.out.println("Agent ordering : " + MaxDegPlusMinDomAO.buildpriorityTree(complexities));
			plotGraphAO();
		}
	}

	/**
	 * 
	 */
	private void plotGraphAO() {
		getEastPanel().add(getGrpahPanel(), GraphAO_PANEL);
		CardLayout cards = (CardLayout) (getEastPanel().getLayout());
		cards.show(getEastPanel(), GraphAO_PANEL);
	}
	/**
	 * 
	 */
	private void plotTreeAO() {
		PriorityTree tree = TreeAO.buildpriorityTree(this.complexities);
		System.out.println("Agent ordering (BFS) : [" + tree.toBFSString()+"]");
		tree.position(0);
		getEastPanel().add(new TreeDrawing(tree, getEastPanel().getSize()), TreeAO_PANEL);
		CardLayout cards = (CardLayout) (getEastPanel().getLayout());
		cards.show(getEastPanel(), TreeAO_PANEL);
	}
}  //  @jve:decl-index=0:visual-constraint="10,10"
