package benchmark.rmsp;
/**
 * this class suppose to represent the distances between places in case there is something else then 
 * Euclidean distance
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 *
 */
public class Geography {
	private double[][] distanceMap;
	private Place[] places;
	private int rows;
	private int columns;

	/**
     *
     */
	public Geography() {
		super();
	}

	public Geography(Place[] places) {
		this.places = places;
		this.setEuclideanDistances();
	}
	
	public Geography(int rows, int columns) {
		this.setRows(rows);
		this.setColumns(columns);
		this.places = new Place[rows * columns];
		for (int r = 0 ; r < rows ; r++) {
			for (int c = 0 ; c < columns ; c++) {
				int id = r * columns + c;
				places[id] = new Place(id, r, c, "place[" + r + "," + c + "]");
			}
		}
		this.setEuclideanDistances();
	}

	private void setEuclideanDistances() {
		this.distanceMap = new double[this.places.length][this.places.length];
		for (int i = 0 ; i < this.distanceMap.length ; i++) {
			for (int j = 0 ; j < this.distanceMap[i].length ; j++) {
				int x1 = this.places[i].getX();
				int y1 = this.places[i].getY();
				int x2 = this.places[j].getX();
				int y2 = this.places[j].getY();
				this.distanceMap[i][j] = Place.distance(x1, y1, x2, y2);
			}
		}
	}

	public String getPlaceName(int p) {
		return this.places[p].getName();
	}

	public Place getPlace(int placeId) {
		return this.places[placeId];
	}

	public Place getRandomPlace() {
		int p = (int) (Math.random() * (this.places.length - 1));
		return this.places[p];
	}

	public double getDistance(int place1, int place2) {
		return this.distanceMap[place1][place2];
	}

	public void setDistance(double[][] distanceMap) {
		this.distanceMap = distanceMap;
	}

	public void setDistance(int place1, int place2, double distance) {
		this.distanceMap[place1][place2] = distance;
	}

	public static Geography generateGrid(int rows, int columns) {
		Place[] places = new Place[rows * columns];
		for (int r = 0 ; r < rows ; r++) {
			for (int c = 0 ; c < columns ; c++) {
				int id = r * columns + c;
				places[id] = new Place(id, r, c, "place[" + r + "," + c + "]");
			}
		}
		return new Geography(places);
	}

	public String toString() {
		String ans = "";
		for (int p = 0 ; p < this.places.length ; p++) {
			ans += this.places[p].pretty() + "\n";
		}
		return ans;
	}

	/**
	 * @param Place
	 * @param Place2
	 * @return
	 */
	public double getDistance(Place place1, Place place2) {
		int p1 = 0;
		int p2 = 0;
		for (int i = 0 ; i < this.places.length ; i++) {
			if (this.places[i].equals(place1)) {
				p1 = i;
			}
			if (this.places[i].equals(place2)) {
				p2 = i;
			}
		}
		return getDistance(p1, p2);
	}

	public void setRows(int rows) {
		this.rows = rows;
	}

	public int getRows() {
		return rows;
	}

	public void setColumns(int columns) {
		this.columns = columns;
	}

	public int getColumns() {
		return columns;
	}
}
