package kernel.constraints.integer;

import java.util.Hashtable;

import kernel.AbstractConstraint;
import kernel.AbstractVar;
import kernel.Variable;
import kernel.vars.integer.IntDomainIterator;
import kernel.vars.integer.IntVariable;
import simple.SConstraint;
import util.IntArray;

public class SUnaryIntConstraint extends AbstractConstraint implements SConstraint {

	static final String VALUES_COSTS = "VALUES_COSTS";

	IntVariable x;

	int C = 0;

	Hashtable<Integer, Integer> costs = null;

	/**
	 * @param var
	 * @param Type
	 * @param cste
	 */
	public SUnaryIntConstraint(String Type, IntVariable var, int cste) {
		this.x = var;
		this.type = Type;
		this.C = cste;
		if (type.equals(VALUES_COSTS))
			throw new Error("Constuctor not compatile with type : " + VALUES_COSTS);
	}

	/**
	 * 
	 * @param var
	 *            the internal variable
	 * @param map
	 *            A map that map each value to a cost
	 */
	public SUnaryIntConstraint(IntVariable var, Hashtable<Integer, Integer> map) {
		this.x = var;
		this.costs = map;
		this.type = VALUES_COSTS;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Constraint#isSatisfied()
	 */
	public boolean isSatisfied() {
		if (!this.isCompletlyInstantiated()) {
			this.incrementNumConstraintChecks(1);
			return true;
		}
		int xval = this.x.getValue();
		return this.isConsistent(xval);
	}

	/**
	 * @param xval
	 *            the value of the internal variable
	 */
	@Override
	public boolean isConsistent(int xval) {
		this.incrementNumConstraintChecks(1);
		if (type.equals(eq))
			return xval == this.C;
		else if (type.equals(neq))
			return xval != this.C;
		else if (type.equals(lt))
			return xval < this.C;
		else if (type.equals(gt))
			return xval > this.C;
		else if (type.equals(VALUES_COSTS))
			return (this.costs.get(this.x.getValue()) == 0);
		return true;

	}

	public void propagate() {
		IntDomainIterator it = this.x.getDomain().iterator();
		while (it.hasNext()) {
			int val = it.next();
			if (!this.isConsistent(val)) {
				// this.x.delete(val, this.index);
				this.x.removeValue(val, this.index);
			}
		}
	}

	@Override
	public void awake() {
		if (this.varEventSources.contains(x.getVarIdentifier())) {
			IntArray evtTypes = x.getEventsType(this.index);
			for (int e : evtTypes) {
				this.awakeOnVariableEvent(x, e);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Constraint#isCompletlyInstantiated()
	 */
	public boolean isCompletlyInstantiated() {
		return this.x.isInstantiated();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see simple.SConstraint#getInternalVar()
	 */
	public IntVariable getInternalVar(int idx) {
		if (idx != 0)
			throw new Error("index : " + idx + " must be equals to zero");
		return this.x;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Constraint#getNbExternalVars()
	 */
	public int getNbExternalVars() {
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Constraint#getNbInternalVars()
	 */
	public int getNbInternalVars() {
		return 1;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Constraint#getNbVars()
	 */
	public int getNbVars() {
		return 1;
	}

	public String toString() {
		return this.x.toString() + this.type + this.C;
	}

	/**
	 * Not used for the unary constraint
	 * 
	 * @throws UnsupportedOperationException
	 */
	public AbstractVar getExternalVar(int indx) {
		throw new UnsupportedOperationException();
	}

	/**
	 * Not used for the unary constraint
	 */
	public void setActiveOnInternalVars() {
	/* noting */
	}

	/**
	 * Not used for the unary constraint
	 */
	public void setPassiveOnInternalVars() {
	/* noting */
	}

	/**
	 * Not used for the unary constraint
	 */
	public void setActiveOnExternalVars() { /* noting */
	}

	/**
	 * Not used for the unary constraint
	 */
	public void setPassiveOnExternalVars() { /* noting */
	}

	@Override
	public void awakeOnVariableEvent(Variable v, int evtType) {
	/* noting */
	}

	@Override
	public boolean isConsistent(Variable v, int val) {
		if (v.equals(x))
			return this.isConsistent(val);
		else
			throw new Error("The given var " + v + " is different from " + x);
	}

	@Override
	public AbstractVar getVar(int i) {
		assert (i == 0);
		return x;
	}

}
