package kernel.order;

import java.io.Serializable;

import util.IntArray;

/**
 * A class to represent the complicity of an agent : number of local variables, of internal constraints, maximum size of domain, the neighbors
 * 
 * @author Redouane EZZAHIR
 * 
 */

public class AgentComplexity implements Serializable {

	private static final long serialVersionUID = -9038560902573312747L;
	
	private int maxDomain;
	private int numberConstraints;
	private int numberLocalVars;
	private IntArray neighbors;
	private IntArray localAgents;
	private int nbNeighbors;

	public AgentComplexity() {
		this.maxDomain = 0;
		this.neighbors = new IntArray();
		this.localAgents = new IntArray();
		this.numberConstraints = 0;
		this.numberLocalVars = 0;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "AgentComplexity [maxDomain=" + this.maxDomain + ", neighbors=" + 
				this.neighbors + ", localAgents=" + this.localAgents + ", numberConstraints=" + 
				this.numberConstraints + ", numberLocalVars=" + this.numberLocalVars + "]";
	}
	
	/**
	 * @return the maxDomain
	 */
	public int getMaxDomain() {
		return this.maxDomain;
	}

	/**
	 * @param maxDomain the maxDomain to set
	 */
	public void setMaxDomain(int maxDomain) {
		this.maxDomain = maxDomain;
	}

	/**
	 * @return the numberConstraints
	 */
	public int getNumberConstraints() {
		return this.numberConstraints;
	}

	/**
	 * @param numberConstraints the numberConstraints to set
	 */
	public void IncrementNbConstraints() { 
		this.numberConstraints++;
	}

	/**
	 * @param numberConstraints the numberConstraints to set
	 */
	public void setNumberConstraints(int numberConstraints) {
		this.numberConstraints = numberConstraints;
	}

	/**
	 * @return the numberLocalVars
	 */
	public int getNumberLocalVars() {
		return this.numberLocalVars;
	}

	/**
	 * @param numberLocalVars the numberLocalVars to set
	 */
	public void setNumberLocalVars(int numberLocalVars) {
		this.numberLocalVars = numberLocalVars;
	}

	/**
	 * @param numberConstraints the numberConstraints to set
	 */
	public void IncrementNbLocalVars() { 
		this.numberLocalVars++;
	}

	/**
	 * @return the neighbors
	 */
	public IntArray getNeighbors() {
		return this.neighbors;
	}

	/**
	 * @param neighbors the neighbors to set
	 */
	public void setNeighbors(IntArray neighbors) {
		this.neighbors = neighbors;
		this.nbNeighbors = neighbors.size();
	}

	/**
	 * @param neighbors the neighbors to set
	 */
	public void addNeighbor(int neighbor) {
		if(!this.neighbors.contains(neighbor)){
			this.neighbors.add(neighbor);
			this.nbNeighbors++;
		}
	}

	/**
	 * @return the localAgents
	 */
	public IntArray getLocalAgents() {
		return this.localAgents;
	}

	/**
	 * @param localAgents the localAgents to set
	 */
	public void setLocalAgents(IntArray localAgents) {
		this.localAgents = localAgents;
	}

	/**
	 * @return the nbNeighbors
	 */
	public int getNbNeighbors() {
		return this.nbNeighbors;
	}
	
}
