package simple.dcsp.abtfamily.abtdoretro;

import static simple.dcsp.abtfamily.abtdoretro.ABTDORetroMaster.After_Second_Last;
import static simple.dcsp.abtfamily.abtdoretro.ABTDORetroMaster.Dom_Pos_All;
import static simple.dcsp.abtfamily.abtdoretro.ABTDORetroMaster.Dom_Pos_Single;
import static simple.dcsp.abtfamily.abtdoretro.ABTDORetroMaster.Highest;
import static simple.dcsp.abtfamily.abtdoretro.ABTDORetroMaster.MinDomain;

import java.io.Serializable;
import java.util.HashMap;

import util.IntArray;

public class CurrentOrder implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1573387988364872830L;
	private IntArray order;
	private HashMap<Integer, Integer> counters;

	public CurrentOrder(IntArray order) {
		this.order = order;
		this.counters = new HashMap<Integer, Integer>();
		for (int agId : this.order) {
			this.counters.put(agId, 0);
		}
	}

	public CurrentOrder(IntArray order, HashMap<Integer, Integer> counters) {
		this.order = order;
		this.counters = counters;
	}

	@SuppressWarnings("unchecked")
	public CurrentOrder clone() {
		HashMap<Integer, Integer> counters = (HashMap<Integer, Integer>) this.counters.clone();
		return new CurrentOrder(this.order.clone(), counters);
	}

	public IntArray getOrder() {
		return this.order;
	}

	public String toString() {
		String order = "[";
		for (int agId : this.order) {
			int counter = this.counters.get(agId);
			order += "(" + agId + "," + counter + ");";
		}
		order = order.substring(0, order.length() - 1);
		order += "]";
		return order;
	}

	// public static CurrentOrder fromString(String str){
	// IntArray order = new IntArray();
	// HashMap<Integer, Integer> counters = new HashMap<Integer, Integer>();
	// str = str.substring(1, str.indexOf("]") );
	// StringTokenizer tok = new StringTokenizer(str,";");
	// while(tok.hasMoreTokens()){
	// String elements = tok.nextToken();
	// elements = elements.substring(1, elements.length()-1);
	// String[] element = elements.split(",");
	// int agId = Integer.parseInt(element[0]);
	// int counter = Integer.parseInt(element[1]);
	// order.add(agId, order.size());
	// counters.put(agId, counter);
	// }
	// return new CurrentOrder(order, counters);
	// }

	public void setNewOrder(int myId, IntArray newOrder) {
		this.order = newOrder;
		IntArray lowerAgents = newOrder.getElementsAfter(myId);
		for (int lower : lowerAgents) {
			this.counters.put(lower, 0);
		}
		this.counters.put(myId, this.counters.get(myId) + 1);
	}

	public void updateCounters(int myId) {
		IntArray lowerAgents = this.order.getElementsAfter(myId);
		for (int lower : lowerAgents) {
			this.counters.put(lower, 0);
		}
		this.counters.put(myId, this.counters.get(myId) + 1);
	}

	/**
	 * 
	 * @param currentOrder
	 * @return <!--
	 * 
	 *         <pre>
	 * 		<b>(i+1)</b> if <i>this</i> order is the newest</br>
	 * 		<b>0</b> if this order and <i>currentOrder</i> are equal</br>
	 * 		<b>-(i+1)</b> if <i>currentOrder</i> is the newest</br>
	 * <!--
	 * </pre>
	 */
	public int compare(CurrentOrder currentOrder) {
		for (int i = 0 ; i < this.order.size() ; i++) {
			int agId1 = this.order.get(i);
			int counter1 = this.counters.get(agId1);
			int agId2 = currentOrder.order.get(i);
			int counter2 = currentOrder.counters.get(agId2);
			if (counter1 > counter2) {
				return (i + 1);
			} else {
				if (counter1 < counter2) {
					return -(i + 1);
				} else {
					if (agId1 > agId2) { return -(i + 1); }
					if (agId1 < agId2) { return (i + 1); }
				}
			}
		}
		return 0;
	}

	/**
	 * public CurrentOrder getNogoodTriggeredOrder(int myId, int agId) { CurrentOrder newOrder = this.clone(); int myRank =
	 * newOrder.order.indexOf(myId); int agRank = newOrder.order.indexOf(agId); for(int i=agRank ; i>myRank+1; i--){
	 * newOrder.order.add(newOrder.order.get(i-1), i); } newOrder.order.add(agId, myRank+1); newOrder.updateCounters(myId); return newOrder; }
	 * 
	 * public CurrentOrder getRandomOrder(int myId) { CurrentOrder newOrder = this.clone(); IntArray lowerAgents =
	 * newOrder.order.getElementsAfter(myId); Random rand = new Random(System.currentTimeMillis()); for(int i=newOrder.order.indexOf(myId)+1 ;
	 * i<newOrder.order.size() ; i++){ int index = rand.nextInt(lowerAgents.size()); int agId = lowerAgents.get(index); lowerAgents.remove(agId);
	 * newOrder.order.add(agId, i); } newOrder.updateCounters(myId); return newOrder; }
	 * 
	 * public CurrentOrder getDomainSizeOrder(int myId, ConcurrentHashMap<Integer, Integer> domainSizes) { CurrentOrder newOrder = this.clone();
	 * IntArray lowerAgents = newOrder.order.getElementsAfter(myId); int[] agents = sort(lowerAgents.clone(), domainSizes); for(int i=0 ;
	 * i<lowerAgents.size() ; i++){ int agId = agents[i]; newOrder.order.add(agId, i+newOrder.order.indexOf(myId)+1); } newOrder.updateCounters(myId);
	 * return newOrder; }
	 * 
	 * public CurrentOrder getSingleRandomOrder(int myId) { CurrentOrder newOrder = this.clone(); Random rand = new
	 * Random(System.currentTimeMillis()); IntArray lowerAgents = newOrder.order.getElementsAfter(myId); int index = rand.nextInt(lowerAgents.size());
	 * int agId = lowerAgents.get(index); int myRank = newOrder.order.indexOf(myId); int agRank = newOrder.order.indexOf(agId); for(int i=agRank ;
	 * i>myRank+1; i--){ newOrder.order.add(newOrder.order.get(i-1), i); } newOrder.order.add(agId, myRank+1); newOrder.updateCounters(myId); return
	 * newOrder; }
	 * 
	 * public CurrentOrder getSingleDomainSizeOrder(int myId, ConcurrentHashMap<Integer, Integer> domainSizes) { CurrentOrder newOrder = this.clone();
	 * IntArray lowerAgents = newOrder.order.getElementsAfter(myId); if(lowerAgents.size()==0){ newOrder.updateCounters(myId); return newOrder; }
	 * int[] agents = sort(lowerAgents.clone(), domainSizes); int agId = agents[0]; int myRank = newOrder.order.indexOf(myId); int agRank =
	 * newOrder.order.indexOf(agId); for(int i=agRank ; i>myRank+1; i--){ newOrder.order.add(newOrder.order.get(i-1), i); } newOrder.order.add(agId,
	 * myRank+1); newOrder.updateCounters(myId); return newOrder; }
	 * 
	 * public CurrentOrder getNewOrder(int myId, int sencondLower) { CurrentOrder newOrder = this.clone(); int agRank =
	 * newOrder.order.indexOf(sencondLower); int agId = newOrder.order.get(agRank+1); int myNewCounter = newOrder.counters.get(agId)+1; int myRank =
	 * newOrder.order.indexOf(myId); for(int i=myRank ; i>agRank+1; i--){ newOrder.order.add(newOrder.order.get(i-1), i); } newOrder.order.add(myId,
	 * agRank+1); IntArray lowerAgents = this.order.getElementsAfter(myId); for(int lower : lowerAgents){ this.counters.put(lower, 0); }
	 * newOrder.counters.put(myId, myNewCounter); return newOrder; }
	 */

	public static int[] sort(IntArray agents, HashMap<Integer, Integer> domainSizes) {
		int[] ans = new int[agents.size()];
		for (int i = 0 ; i < ans.length ; i++) {
			int minId = agents.get(0);
			int minSize = domainSizes.get(minId);
			for (int agId : agents) {
				if (minSize > domainSizes.get(agId)) {
					minSize = domainSizes.get(agId);
					minId = agId;
				}
			}
			ans[i] = minId;
			agents.remove(minId);
		}
		return ans;
	}

	public boolean equals(Object obj) {
		CurrentOrder arg = (CurrentOrder) obj;
		return (arg.order.compare(this.order) == 0 && this.compare(arg) == 0);
	}

	public HashMap<Integer, Integer> getCounters() {
		return this.counters;
	}

	/**
	 * 
	 * @param heuristic
	 * @param myId
	 * @param lower
	 * @param lower2
	 * @param D
	 * @return
	 */
	public CurrentOrder getNewOrder(short heuristic, int agentId, int target, int secondlower, HashMap<Integer, Integer> domainSizes) {
		switch (heuristic) {
			case MinDomain :
				return this.getMinDomainOrder(agentId, target, secondlower, domainSizes);
			case After_Second_Last :
				return this.getAfterOrder(agentId, secondlower);
			case Dom_Pos_Single :
				return this.getPositionSingleOrder(agentId, target, secondlower, domainSizes);
			case Dom_Pos_All :
				return this.getPositionAllOrder(agentId, target, secondlower, domainSizes);
			case Highest :
				return this.getHighestOrder(agentId, target, secondlower, domainSizes);
		}
		return null;
	}

	private CurrentOrder getHighestOrder(int agentId, int target, int secondlower, HashMap<Integer, Integer> domainSizes) {
		// TODO Auto-generated method stub
		return null;
	}

	private CurrentOrder getPositionAllOrder(int agentId, int target, int secondlower, HashMap<Integer, Integer> domainSizes) {
		// TODO Auto-generated method stub
		return null;
	}

	private CurrentOrder getPositionSingleOrder(int agentId, int target, int secondlower, HashMap<Integer, Integer> domainSizes) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * 
	 * @param myId
	 * @param agId
	 * @return
	 */
	public CurrentOrder getAfterOrder(int myId, int agId) {
		CurrentOrder newOrder = this.clone();
		int myRank = newOrder.order.indexOf(myId);
		int agRank = newOrder.order.indexOf(agId);
		int afterRank = newOrder.order.get(agRank + 1);
		int myNewCounter = newOrder.counters.get(afterRank) + 1;
		IntArray lowerAgents = newOrder.order.getElementsAfter(myId);
		for (int lower : lowerAgents) {
			newOrder.counters.put(lower, 0);
		}
		for (int i = myRank ; i > agRank + 1 ; i--) {
			int id = newOrder.order.get(i - 1);
			newOrder.order.add(id, i);
			newOrder.counters.put(id, 0);
		}
		newOrder.order.add(myId, agRank + 1);
		newOrder.counters.put(myId, myNewCounter);
		return newOrder;
	}

	/**
	 * 
	 * @param myId
	 * @param lower
	 * @param secondlower
	 * @param D
	 * @return
	 */
	private CurrentOrder getMinDomainOrder(int myId, int lower, int secondlower, HashMap<Integer, Integer> D) {
		CurrentOrder newOrder = this.getAfterOrder(myId, lower);
		int secondRank = newOrder.order.indexOf(secondlower);
		int myRank = newOrder.order.indexOf(myId);
		int myD = D.get(myId);
		for (int i = myRank ; i > secondRank + 1 ; i--) {
			int agId = newOrder.order.get(i - 1);
			int agD = D.get(agId);
			int agCounter = this.counters.get(agId);
			if (myD < agD) {
				newOrder.order.add(agId, i);
				newOrder.counters.put(agId, 0);
				newOrder.order.add(myId, i - 1);
				newOrder.counters.put(myId, agCounter + 1);
			} else {
				break;
			}
		}
		return newOrder;

	}

	public static void main(String[] agrs) {
		IntArray order = new IntArray();
		order.add(1);
		order.add(2);
		order.add(3);
		order.add(4);
		order.add(5);
		HashMap<Integer, Integer> domainSizes = new HashMap<Integer, Integer>();
		domainSizes.put(1, 5);
		domainSizes.put(2, 2);
		domainSizes.put(3, 3);
		domainSizes.put(4, 4);
		domainSizes.put(5, 1);
		System.out.println(order + "\n" + domainSizes);
		int[] agents = order.getArray();
		for (int i = 0 ; i < agents.length ; i++) {
			System.out.print(agents[i] + " ");
		}
		System.out.println();
		agents = sort(order, domainSizes);
		for (int i = 0 ; i < agents.length ; i++) {
			System.out.print(agents[i] + " ");
		}
		System.out.println();
	}

}
