package simulation;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Scanner;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;

import kernel.VarIdentifier;
import kernel.order.AgentOrdering;
import util.Commands;
import util.Instantiation;
import util.Tools;
import util.parser.InstanceParser;
import util.parser.PConstraint;
import util.parser.PExtensionConstraint;
import util.parser.PVariable;
import dischoco.Protocols;

/**
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr"> Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr"> Mohamed Wahbi</a>
 */
public class Main {
	
	/** nohup java -cp dischoco.jar simulation.Experimentation RMSP/afc.xml > msafc.txt & */
	private static final String Layout = "/usr/local/bin/dot";
	/**
	 * @param args
	 */
	public static void main(String[] args) {
//		util.Logger.LOG_DEBUG = true;
//		util.Logger.log_debug_level = util.Logger.INFO_LEVEL;
		int nbExecution = 5;
		long maxNCCCs = (long) 1.0E7; //10000000
		boolean solvable = true;
		String protocol = "" ;
		String problem = "" ;
		String ordering = "";
//		ordering = Commands.LexicoAO;
		ordering = Commands.RandomAO;
//		ordering = Commands.MinDomDegAO;
//		ordering = Commands.MaxDegPlusMinDomAO;
//		ordering = Commands.TreeAO;
		
//		protocol = Protocols.AFCngDomDeg;
//		protocol = Protocols.AFCngMinDom;
//		protocol = Protocols.AFC;
		protocol = Protocols.AFCng;
		protocol = Protocols.AFCngTriggeredng;
//		protocol = Protocols.AFCtree;
//		protocol = Protocols.DDBJ;
//		protocol = Protocols.AFCdbj;
//		protocol = Protocols.AFCngApproximatedMinDom;
//		protocol = Protocols.AFCngApproximatedDomDeg;
		problem = "./samples/RandomDCSP/p1_0.70/0.30/RandomTKC_20_10_1.xml.bz2";
//		problem = "./samples/RandomDCSP/p1_0.20/0.35/RandomTKC_20_10_12.xml.bz2";
//		problem = "./samples/RandomDCSP/p1_0.20/0.70/RandomTKC_20_10_1.xml.bz2";
//		problem = "./samples/nqueens/ext/NQueen_6.xml.bz2";
//		problem = "./samples/GSN/25/Kc_2-Kv_2/Pc_0.40/0.15/GSensorDCSP_25_5_6.xml.bz2";
//		problem = "./samples/RMSP/M-9_A-20/3/6/MSP_5.xml.bz2";
//		problem = "./samples/GSN/25/Kc_2-Kv_2/Pc_0.40/0.15/GSensorDCSP_25_5_18.xml.bz2";
		
//		//random triggered 
		problem = "./samples/GSN/25/Kc_2-Kv_2/Pc_0.40/0.10/GSensorDCSP_25_5_3.xml.bz2";

		//FIXME
//		test(problem);
		run(problem, protocol, ordering, nbExecution, solvable, maxNCCCs);
	}
	
	/**
	 * 
	 * @param problem
	 * @param protocol
	 * @param nbExecution
	 * @param solvable
	 */
	protected static void run(String problem, String protocol, String ordering, int nbExecution, boolean solvable, long maxNCCCs) {
		for (int i = 1 ; i <= nbExecution ; i++) {
			DisProblem disProb = new DisProblem(problem, protocol);
			DisSolverSimulator solver = new DisSolverSimulator(disProb);
			solver.addCommunicationMetric();
			solver.setCutOffNCCCs(maxNCCCs);
//			solver.setSlowCommunication();
//			solver.CheckTheReturnedSolution(true);
			solver.setCentralizedAO(AgentOrdering.getCentralizedAO(ordering));
			solver.solve();
			if (solver.getEndState().equals(Commands.SUCCESS)) {
				String s = "------------------------------------------------";
				System.err.println(i + " " + protocol + " with "+ordering+ "\n" + s + s + s + s);
				Instantiation solution = solver.getSolution();
				System.err.println(solution);
				HashMap<String, Double> runStatistics = solver.getStatistics();
				ArrayList<String> keys = disProb.master.getStatisticsKeys();
				System.err.println(Tools.toString(keys) + "\n" + Tools.statisticsToString(keys, runStatistics));
				System.err.println(s + s + s + s + "\n");
				if (!solvable) {
					break;
				}
			} else {
				if (solver.getEndState().equals(Commands.FAILURE)) {
					String s = "*************************************************";
					System.err.println(i + " " + protocol + " with "+ordering+ "\n" + s + s + s + s);
					System.err.println(solver.getGlobalSolution());
					HashMap<String, Double> runStatistics = solver.getStatistics();
					ArrayList<String> keys = disProb.master.getStatisticsKeys();
					System.err.println(Tools.toString(keys) + "\n" + Tools.statisticsToString(keys, runStatistics));
					System.err.println(s + s + s + s + "\n");
					if (solvable) {
						break;
					}
				} else {
					String s = "================================================";
					System.err.println(i + " " + protocol + " with "+ordering+ "\n" + s + s + s + s);
					System.err.println(solver.getGlobalSolution());
					HashMap<String, Double> runStatistics = solver.getStatistics();
					ArrayList<String> keys = disProb.master.getStatisticsKeys();
					System.err.println(Tools.toString(keys) + "\n" + Tools.statisticsToString(keys, runStatistics));
					System.err.println(s + s + s + s + "\n");
					break;
				}
			}
		}
	}

	public static boolean isSolution(String problemFile, Instantiation solution) {
		InstanceParser parser = new InstanceParser(problemFile);
		parser.loadInstance(problemFile);
		parser.parse(false);
		Map<String, PConstraint> constraints = parser.getMapOfConstraints();
		Iterator<String> it = constraints.keySet().iterator();
		while (it.hasNext()) {
			PConstraint constraint = constraints.get(it.next());
			PVariable[] scope = constraint.getScope();
			int agId1 = scope[0].getAgent().getId();
			int agId2 = scope[1].getAgent().getId();
			int val1 = solution.getVarAssignment(new VarIdentifier(agId1)).getValue();
			int val2 = solution.getVarAssignment(new VarIdentifier(agId2)).getValue();
			int[][] tuples = ((PExtensionConstraint) constraint).getRelation().getTuples();
			for (int i = 0 ; i < tuples.length ; i++) {
				int v1 = tuples[i][0];
				int v2 = tuples[i][1];
				if (val1 == v1 && val2 == v2) {
					String s = "================================================";
					System.err.println("\n" + s + s + s + s);
					System.err.println("The constraint " + constraint.getName() + " is not satisfied " + " \t==>  " + agId1 + "=" + val1 + " <> "
							+ agId2 + "=" + val2);
					System.err.println(solution);
					System.err.println(s + s + s + s + "\n");
					return false;
				}
			}
		}
		return true;
	}
	/**
	 * 
	 * @param pb
	 * @param path
	 * @param name
	 */
	public static void toCSPXML(String pb, String path, String name){
		InstanceParser parser = new InstanceParser(pb);
		parser.loadInstance(pb);
		parser.parse(false);
		parser.convertToCSPModel();
		parser.saveXML(path, name);
	}

	/**
	 * @param problem
	 */
	protected static void test(String problem) {
		JFrame frame = new JFrame();
		JScrollPane center = null;	
		JLabel content = null;
		frame.setTitle("DisChoco: Visualizing the meeting scheduling problem");
		InstanceParser parser = new InstanceParser(problem);
		parser.loadInstance(problem);
		parser.parse(false);
		String fileName = problem.substring(0, problem.length()-4)+".dot";
		String dotCode = parser.toDOT();
		File dotFile = new File(fileName);
		try {
			PrintWriter writer = new PrintWriter(dotFile);
			writer.println(dotCode);
			writer.close();
//			dotFile.deleteOnExit();	//Assures the temporary file is deleted when JVM exits
		} 
		catch (FileNotFoundException e) {
			System.err.println("Could not create DOT output file with name " + fileName);
			e.printStackTrace();
		}
		ImageIcon currentDotImage = createDOTfigure(fileName);

		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		//Image could not be created; an error message is shown in the Frame
		if(currentDotImage == null) {
			content = new JLabel("Could not create graphic from .dot file");
			frame.setSize(dim.width/2, dim.height/2);
		}

		//Image is displayed as a content of the label; Frame size is set according to the size of the image
		else {
			content = new JLabel(currentDotImage);
			int width = Math.max(300, Math.min(dim.width - 100, currentDotImage.getIconWidth() + 50));
			int height = Math.max(100, Math.min(dim.height - 150, currentDotImage.getIconHeight() + 100));
			frame.setSize(width, height);
		}
		// Determine the new location of the window
		int w = frame.getSize().width;
		int h = frame.getSize().height;
		int x = (dim.width - w) / 2;
		int y = (dim.height - h) / 2;
		// Move the window
		center = new JScrollPane(content);
		frame.add(center, BorderLayout.CENTER);
		frame.setLocation(x, y);
		frame.pack();
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}
	/**
	 * 
	 * @param dotFileName
	 * @return
	 */
	private static ImageIcon createDOTfigure(String dotFileName) {
		ImageIcon dotImageIcon = null;
		int status = -1;
		File input = new File(dotFileName);
		try {
			Process p = Runtime.getRuntime().exec(new String[] {Layout, "-Tgif", input.getAbsolutePath(), "-O"});
			final Scanner err = new Scanner(p.getErrorStream());
			new Thread(new Runnable() {
				public void run() {
					while(err.hasNextLine()) {
						System.err.println(err.nextLine());
					}
				}
			}).start();
			status = p.waitFor();	//0 if creation of image was successful
		} 
		catch (IOException e) {
			if(e.getMessage().contains("Cannot run program")) {
				System.err.println("Can not run \"" + Layout + "\". Probably, the program is not installed, or it is not on the search path.");
			}
			else {
				e.printStackTrace();
			}
		} catch (InterruptedException e) {}

		if(status == 0) {
			dotImageIcon = new ImageIcon(dotFileName + ".gif");
			//Image needs to be stored, as scaling should be done from this image in order not to loose too much quality
//			Image dotImage = dotImageIcon.getImage();
			new File(dotFileName + ".gif").deleteOnExit();	//Assures the temporary .gif file will be deleted when JVM exits
		}
		
		try {
			Process p = Runtime.getRuntime().exec(new String[] {Layout, "-Tpdf", input.getAbsolutePath(), "-O"});
			final Scanner err = new Scanner(p.getErrorStream());
			new Thread(new Runnable() {
				public void run() {
					while(err.hasNextLine()) {
						System.err.println(err.nextLine());
					}
				}
			}).start();
			status = p.waitFor();	//0 if creation of image was successful
		} 
		catch (IOException e) {
			if(e.getMessage().contains("Cannot run program")) {
				System.err.println("Can not run \"" + Layout + "\". Probably, the program is not installed, or it is not on the search path.");
			}
			else {
				e.printStackTrace();
			}
		} catch (InterruptedException e) {}
		return dotImageIcon;
	}
}
