package simulation.metric;

import java.lang.management.ManagementFactory;
import java.lang.management.ThreadMXBean;
import java.util.Collection;

import kernel.AgentObserver;
import kernel.StopException;
import kernel.com.Message;
import kernel.events.AgentEvent;
import util.Commands;

/**
 * A class for measuring the Non-Concurrent Computation-Time
 * 
 * @author Redouane EZZHIR
 */
public class NCCTobserver extends AgentObserver implements MetricListener {

	/**
	 * The current Non-Concurrent Computation-Time
	 */
	long NCCT;
	/**
	 * the starting time
	 */
	long startTime;
	/**
	 * The moment when the agent moves from a pending state message to the message processing state (i.e when the agent becomes active)
	 */
	long currentBeginTime;
	/**
	 * Used to returns the total CPU time for the current thread in nanoseconds.
	 */
	private ThreadMXBean tmxb = null;

	/**
	 * constructs an observer
	 */
	public NCCTobserver() {
		super(Commands.NCCT);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void initialize() {
		super.initialize();
		this.NCCT = 0;
		this.currentBeginTime = -1;
		this.tmxb = ManagementFactory.getThreadMXBean();
		this.myAgent.addAgentEventListener(this);
		this.startTime = this.tmxb.getCurrentThreadCpuTime();
	}

	/**
	 * Updates the current Non-Concurrent Computation-Time to be the maximum between its value and the value attached in the message <code>msg</code>
	 */
	@Override
	public boolean handleReceivedMessage(Message msg) throws StopException {
		long senderCTcost = msg.getNCCT();
		this.NCCT = Math.max(this.NCCT, senderCTcost);
		return true;
	}

	/**
	  * 
	  */
	@Override
	public void handleSendMessage(Message msg) {
		if (this.currentBeginTime != -1) {
			this.NCCT += (tmxb.getCurrentThreadCpuTime() - this.currentBeginTime) / 1000;
			this.currentBeginTime = tmxb.getCurrentThreadCpuTime();
		} else{
			throw new Error("In Non-Concurrent Computation-Time !");
		}
		if (this.myAgent != null){
			msg.setNCCT(NCCT);
		}
	}

	@Override
	public void handleAgentEvent(AgentEvent event) {
	// if (event instanceof AgentChangeStateEvent) {
	// if (((AgentChangeStateEvent) event).getAgentState() == AgentChangeStateEvent.STATE_RUNNING) {
	// this.currentBeginTime = tmxb.getCurrentThreadCpuTime();
	// } else if (((AgentChangeStateEvent) event).getAgentState() == AgentChangeStateEvent.STATE_WAITING) {
	// this.NCCT += (tmxb.getCurrentThreadCpuTime() - this.currentBeginTime) / 1000;
	// this.currentBeginTime = -1;
	// }
	// } else if (event instanceof AgentFinishedEvent) {
	// long tcput = tmxb.getCurrentThreadCpuTime() / 1000000;
	// Message msg = new Message(Commands.STATISTICS);
	// msg.addContents(Commands.KEY, Commands.NCCT);
	// msg.addContents(Commands.VALUE, (this.NCCT / 1000));
	// msg.addContents(Commands.OPERATOR, Commands.MAX);
	// msg.addContents("Total CPU time", tcput);
	//
	// msg.setReceiverId(AbstractMaster.MasterId);
	// this.myAgent.sendMessage(msg);
	// // this.myAgent.log("Non-Concurrent computation-time : " + this.NCCT
	// // / 1000 + " ms", Logger.INFO_LEVEL);
	// // this.myAgent.log("Total CPU time : " + tcput + " ms ",
	// // Logger.INFO_LEVEL);
	// }

	}

	/**
	 * not used for this metric
	 */
	@Override
	public void handleIncrementNbConstraintChecks(int n) {}

	/**
	 * not used for this metric
	 */
	@Override
	public void handleIncrementCycle() {}

	public MetricListener clone() {
		MetricListener clone = null;
		try {
			clone = (NCCTobserver) super.clone();
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return clone;
	}

	@Override
	public Collection<Metric> getEndStatistics() {
		// TODO Auto-generated method stub
		return null;
	}

}
